/*
 * Created on 1-feb-2005
 *
 */
package it.unibo.txs.app;

import it.unibo.txs.prng.PrngControl;
import it.unibo.txs.prntest.TestControl;
import it.unibo.txs.util.DialogUtil;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;

/**
 * @author patrik.suzzi@gmail.com
 *
 * This class is the control class for the main application
 */
public class AppControl implements ActionListener {
	
	protected AppJFrame appJFrame;
	protected AppModel appModel;
	protected TestControl testControl ;
	protected PrngControl prngControl ;
	
	public void startApp(){
		this.appJFrame.setVisible(true);
	}
	
	/**
	 * Creates new instance of AppControl
	 *
	 */
	public AppControl(){
		// initialize the view
		this.appJFrame = new AppJFrame( this );
		// initialize the model
		this.appModel = new AppModel();
		// initialize child controls
		this.testControl = new TestControl( 
				appModel.getTestModel(),
				appJFrame.getTestConsole() );
		this.prngControl = new PrngControl( 
				appModel.getPrngModel(), 
				appJFrame.getPrngConsole() );
		// initialize view(s)
		this.appModel.getPrngModel().addObserver(
				this.appJFrame.getAppTextView()
			);
		this.appModel.getTestModel().addObserver(
				this.appJFrame.getAppTextView1()
			);
		this.appModel.addObserver(
				this.appJFrame.getAppTextView()
				);
	}

	/**
	 * Executes the action specified by the command
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent evt) {
		String cmd = evt.getActionCommand();
		String src = evt.getSource().getClass().toString();
		if( Settings.debug ){
			System.out.println( "AppControl <-- " + cmd + " ( " + src + " )" );
		}
		if( cmd.equalsIgnoreCase("load") ){
			load();
			
		}
		if( cmd.equalsIgnoreCase("save") ){
			save();
			
		}
		if( cmd.equalsIgnoreCase("test") ){
			test();
			
		}
		if( cmd.equalsIgnoreCase("generate") ){
			generate();
			
		}
	}
	



	/**
	 * 
	 */
	private void generate() {
		appModel.setSample( this.prngControl.generate() );
		
	}

	/**
	 * 
	 */
	private void test() {
		if( this.appModel.getSample() != null )
			this.testControl.test( appModel.getSample() );
		else
			DialogUtil.showWarnMsg( this.appJFrame, "It's better to set a sample to test" );
		
	}

	/**
	 * 
	 */
	private void save() {
		byte[] sample = appModel.getSample();
		if( Settings.debug )
			System.out.println( "Saving sample (" + sample.length + " Bytes )" );
		DialogUtil.saveFileBytes( sample, this.appJFrame );
	}

	/**
	 * 
	 */
	private void load() {
		byte[] fileBytes = DialogUtil.loadFileBytes( this.appJFrame );
		if( Settings.debug )
			System.out.println( "Loading fileBytes (" + fileBytes.length + " Bytes )" );
		// setting up the model
		if( fileBytes.length == 2500 )
			appModel.setSample( fileBytes );
		else{
			DialogUtil.showErrorMsg( this.appJFrame, "Problems in Loading" );
		}
	}

	/**
	 * This method receive the windowClosing event then closes the application
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent arg0) {
		if( Settings.debug )
			System.out.println("windowClosing()");
		System.exit( 0 );
		
	}


}
