/*
 * Created on 1-feb-2005
 *
 */
package it.unibo.txs.prng;

import it.unibo.txs.prng.gens.PrngBBS;
import it.unibo.txs.prng.gens.PrngRandom;
import it.unibo.txs.prng.gens.PrngSecureRandom;
import it.unibo.txs.prng.gens.PrngX917;

import java.security.SecureRandom;
import java.util.Observable;

/**
 * @author patrik.suzzi@gmail.com
 *
 * This class represents the model for the prng.
 */
public class PrngModel extends Observable {
	
	public static final String SECURE_RANDOM 	= "SecureRandom";
	public static final String RANDOM 			= "Random";
	public static final String BBS 				= "bbs";
	public static final String X917 			= "x917";
	
	protected byte[] seed;
	protected String prngName;
	protected SecureRandom prng;
	
	/**
	 * Creates new instance of PrngModel
	 *
	 */
	public PrngModel(){
		// initialize prng
		this.seed = new byte[2500];
		prngName = "SecureRandom";
		prng = new SecureRandom();
		// fills the seed with pseudo-random bytes
		prng.nextBytes(seed);

	}
	
	/**
	 * This method set a new prng for generating next bytes.
	 * BE CAREFUL: "Random" is NOT a SecureRandom, but is used as
	 * it for learning purposes.
	 * Valid names
	 * @param prngName the name of the prng
	 */
	public void setPrngName( String prngName ){
		// says that the model has changed
		this.setChanged();
		// watch out: Random is not a SecureRandom !
		if( prngName.equalsIgnoreCase("random") ){
			this.prngName = prngName;
			prng = new PrngRandom();
		}
		if( prngName.equalsIgnoreCase("securerandom") ){
			this.prngName = prngName;
			prng = new PrngSecureRandom();
		}
		if( prngName.equalsIgnoreCase("bbs") ){
			this.prngName = prngName;
			prng = new PrngBBS();
		}
		if( prngName.equalsIgnoreCase("x917") ){
			this.prngName = prngName;
			prng = new PrngX917();
		}
		// uses the same seed for generators
		prng.setSeed( this.seed );
		// notify all the observers
		this.notifyObservers();
	}

	/**
	 * @return Returns the prng.
	 */
	public SecureRandom getPrng() {
		return prng;
	}

	/**
	 * @return Returns the prngName.
	 */
	public String getPrngName() {
		return prngName;
	}
}
