/*
 * PrngX917.java
 *
 * Created on 6 marzo 2004, 13.05
 */

package it.unibo.txs.prng.gens;

import java.security.*;
import javax.crypto.*;
import java.util.*;

/**
 *
 * @author  x0104993
 */
public class PrngX917 extends SecureRandom{
  
  static final int SEED_DIM = 8; //dimensione in bytes del seme
  private byte[] seed;
  private byte[] inizio =new byte[8];
  
  /** Creates a new instance of RNG */
  public PrngX917() {
    //this(null);
    this.seed = new byte[SEED_DIM];
    Random r = new Random();
    r.nextBytes(this.seed);
    r.nextBytes(inizio);
  }
  
  /** Crea un nuovo PRNG, dato il valore del seme (seed). se tale valore
   *  nullo oppure la lunghezza  inferiore a quella del seme creo un seme
   * ex novo con la classe java.util.Random
   *(OCCHIO CHE java.util.Random passa a malapena i 4 test base)
   */
  public PrngX917(byte[] seed) {
    //if( (seed != null) || (seed.length == this.SEED_DIM)) {
    this.seed = seed;
    Random r = new Random();
    r.nextBytes(inizio);
    //}
    //else // il seed passato  nullo o la dimensione  sbagliata.
    /*{
      this.seed = new byte[SEED_DIM];
      Random r = new Random();
      r.nextBytes(this.seed);
      r.nextBytes(inizio);
    }*/
  }
  
  public synchronized void setSeed(byte[] seed){
    this.seed = seed;
  }
  
  /** riempie l'array di byte passato dall'utente
   */
  
  //bytes deve avere grandezza di
  public synchronized void nextBytes(byte [] bytes) {
    //System.out.println("\nRiempo l'array...!!");
    //Caso 1: l'utente ha passato un array nullo
    if (bytes==null)
      return;
    else{
      //Altrimenti riempo i byte che desidera l'utente di valori casuali
      byte[][] RandNum;
      byte[] PRandBytes;
      RandNum = byteToBlocchi(bytes, 8);
      
      int NumIterazioni = (bytes.length-1) / 8 +1;
      //System.out.println("\nNumero di blocchi:"+ NumIterazioni);
      
      for(int i =0; i< NumIterazioni; i++){
        //System.out.println("\nDentro al ciclo....numero: " + i);
        try {
          
          //creazione chiavi
          KeyGenerator keyGen = KeyGenerator.getInstance("TripleDES");
          keyGen.init(168);    // obbligatorio chiave di 168 bit con TDES!!
          Key key1 = keyGen.generateKey();
          Key key2 = keyGen.generateKey();
          //System.out.println("\nChiavi generate!!");
          
          // prima cifratura TDES
          Cipher cipher = Cipher.getInstance("TripleDES/ECB/PKCS5Padding");
          cipher.init(Cipher.ENCRYPT_MODE,key1);
          byte[] cifrato1 =new byte[8];
          cifrato1 = cipher.doFinal(inizio);
          //System.out.println("\nPrimo blocco cifrato!!");
          
          //prima somma modulo 2
          byte[] somma1= new byte[8];
          for(int j=0;j<8;j++)
            somma1[j] = (byte) (cifrato1[j]^seed[j]);
          
          
          
          // seconda cifratura TDES
          cipher.init(Cipher.ENCRYPT_MODE,key2);
          byte[] cifrato2 =new byte[8];
          cifrato2 = cipher.doFinal(somma1);
          //System.out.print("Secondo blocco cifrato!!");
          
          //cifrato2 rappresenta i 8 byte pseudo-casuali (r) !!!!
          
          for(int k=0; k < PiuPiccolo(RandNum[i].length,8); k++)
            RandNum[i][k] = cifrato2[k];
          //System.out.println("\n" + RandNum[0]);
          
          
          //seconda somma modulo 2
          byte[] somma2= new byte[8];
          for(int k=0;k<8;k++)
            somma2[k] = (byte) (cifrato2[k]^cifrato1[k]);
          
          
          // terza cifratura TDES x estrarre il nuovo seed
          cipher.init(Cipher.ENCRYPT_MODE,key1);
          byte[] cifrato3 =new byte[8];
          cifrato3 = cipher.doFinal(somma2);
          //System.out.println("\nTerzo blocco cifrato!!");
          
          
          //cifrato3 rappresenta il nuovo seme (s+1)!!!
          System.arraycopy(cifrato3, 0, seed, 0 , 8);
          //seed = (byte[]) cifrato3.clone();
          
        }//try
        catch(Exception e) {
          String  res = "Si  verificata un'eccezione.";
          //System.out.println(res);
        }
        
      }//for
      
      //Array di Bytes riempito di valori pseudo-casuali!!!
      PRandBytes = blocchiToByte(RandNum);
      
           /*DEBUG
           for(int i = 0; i < PRandBytes.length; i++)
           System.out.print("" +  PRandBytes[i] + "\n");
            */
      
      
      System.arraycopy(PRandBytes, 0, bytes, 0 , bytes.length);
      
      return;
      
    }//else
  }
  
  /**
   * Converte un array di byte nel corrispondente array di blocchi di byte. I blocchi di
   * byte hanno lunghezza (in byte) specificata dal parametro numBytePerBlocco. Se il vettore
   * di byte non contiene esattamente un numero di elementi multiplo di numBytePerBlocco,
   * allora l'ultimo blocco ha un numero di byte inferiore.
   *
   * @param arrayByte array di byte da convertire
   * @param numBytePerBlocco dimensione in byte di un singolo blocco
   *
   * @return array di blocchi della dimensione specificata contenente i byte di ingresso
   */
  public static byte[][] byteToBlocchi(byte[] arrayByte, int numBytePerBlocco) {
    int dimArrayBlocchi = (arrayByte.length-1) / numBytePerBlocco +1,
    indiceArrayByte = 0;
    
    byte[][] arrayBlocchi = new byte[dimArrayBlocchi][];
           /* arrayBlocchi = array di dimArrayBlocchi elementi i quali sono array
              ancora da allocare  */
    
    int i,j; // contatori per i cicli
    
        /* ciclando da 0 a dimArrayBlocchi-1 (tutti i blocchi tranne l'ultimo) si  sicuri che
           tutti i blocchi da creare devono avere esattamente numBytePerBlocco byte ciascuno,
           infatti se la dimensione di arrayByte non  multipla di numBytePerBlocco solo
           l'ultimo blocco da allocare ha meno di numBytePerBlocco byte */
    
    for (i=0; i < dimArrayBlocchi-1; i++) {
      // scansione dei blocchi: si creano e si riempiono
      
      arrayBlocchi[i] = new byte[numBytePerBlocco];
      
      for (j=0; j < numBytePerBlocco; j++, indiceArrayByte++) {
                /* ad ogni iterazione di questo ciclo interno si pone un byte di arrayByte
                   dentro il blocco i selezionato dal ciclo esterno*/
        
        arrayBlocchi[i][j] = arrayByte[indiceArrayByte];
        
                /* il byte con posizione indiceArrayByte in arrayByte  collocato nel byte in
                    posizione j del blocco in posizione i */
      }
    }
    
    i = dimArrayBlocchi-1; // assegnazione inutile
    
    arrayBlocchi[i] = new byte[arrayByte.length-indiceArrayByte];
    
    for (j=0; j < arrayBlocchi[i].length; j++, indiceArrayByte++) {
      arrayBlocchi[i][j] = arrayByte[indiceArrayByte];
      
          /* il byte con posizione indiceArrayByte in arrayByte  collocato nel byte in
             posizione j del blocco in posizione i */
    }
    return arrayBlocchi;
  }
  
  /**
   * Converte un array di Blocchi di byte in un array di byte. Questo metodo  il reciproco
   * del metodo byteToBlocchi. La dimensione di un singolo blocco  estratta direttamente
   * dall'array di blocchi, quindi non  necessaria. Se i blocchi in arrayBlocchi non sono
   * tutti allocati allora il metodo ritorna in uscita un vettore non allocato. Se i blocchi
   * hanno dimensioni diverse il metodo funziona ugualmente.
   *
   * @param arrayBlocchi array di blocchi da convertire in byte
   *
   * @return array di byte estratti dai blocchi
   */
  
  public static byte[] blocchiToByte(byte[][] arrayBlocchi) {
    
    if (arrayBlocchi == null ) return null;
    /* se non arrayBlocchi non  allocato, allora non si alloca nulla */
    
    int i,                 // contatore
    dimArrayByte = 0;  // accumulatore
    
    for (i = 0; i < arrayBlocchi.length; i++) {
      if (arrayBlocchi[i] == null) return null;
      /* se un blocco non  allocato allora si esce con un vettore non allocato*/
      
      dimArrayByte = dimArrayByte+arrayBlocchi[i].length;
            /* si accumulano le lunghezza di tutti i blocchi (che non necessariamente hanno
               la stessa lunghezza)*/
    }
    int indiceArrayByte = 0; // indice
    
    byte[] arrayByte = new byte[dimArrayByte];
          /* si crea un vettore di byte con dimensione sufficiente per copiarci dentro
             tutti i byte contenuti nei blocchi do arrayBlocchi */
    
    int j; // contatore
    
    for (i=0; i < arrayBlocchi.length; i++) {
      for (j=0; j < arrayBlocchi[i].length; j++, indiceArrayByte++) {
        arrayByte[indiceArrayByte] = arrayBlocchi[i][j];
        // copia del singolo byte
      }
    }
    
    return arrayByte;
  }
  
  
  public static int PiuPiccolo(int a, int b) {
    if(a < b) return a;
    else return b;
  }

 
  
  
}
