/*
 * Created on 1-feb-2005
 *
 */
package it.unibo.txs.prntest;


import it.unibo.txs.app.Settings;
import it.unibo.txs.prntest.tests.LongRunTest;
import it.unibo.txs.prntest.tests.MonobitTest;
import it.unibo.txs.prntest.tests.PokerTest;
import it.unibo.txs.prntest.tests.PrngTest;
import it.unibo.txs.prntest.tests.RunsTest;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;

/**
 * @author patrik.suzzi@gmail.com
 *
 * This class represents the control for prn tests.
 */
public class TestControl implements ActionListener {
	
	protected TestModel testModel;
	protected TestConsole testConsole;
	
	public TestControl( TestModel testModel, TestConsole testConsole ){
		this.testModel = testModel;
		this.testConsole = testConsole;
		this.testConsole.addListener( this );
		this.testConsole.initialize( testModel );
	}

	/**
	 * Executes the action specified by the command
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent evt) {
		String cmd = evt.getActionCommand();
		if( Settings.debug ){
			System.out.println( "PrngControl <-- " + cmd );
		}
		if(	cmd.equalsIgnoreCase("longrun") ||
			cmd.equalsIgnoreCase("monobit") ||
			cmd.equalsIgnoreCase("poker") ||
			cmd.equalsIgnoreCase("runs") ){
			// selection of what PRNG to use
			
			testModel.setTest(
					cmd, 
					( (JCheckBox) evt.getSource() ).isSelected() 
					);
		}
	}
	
	/**
	 * this method executes the test(s) specified
	 *
	 */
	public void test( byte[] sample ){
		PrngTest test = null;
		testModel.setTestResults( "" );
		
		boolean failed = false;
		// test Selection
		for( int i=0; i<TestModel.TEST_NAMES.length; i++ ){
			String testName = TestModel.TEST_NAMES[i];
			if( testModel.isTestSelected( testName ) ){
				// Specific test istantiation
				if( testName.equalsIgnoreCase("longrun") ){
					test = new LongRunTest();
				}
				else if( testName.equalsIgnoreCase("monobit") ){
					test = new MonobitTest();
				}
				else if( testName.equalsIgnoreCase("poker") ){
					test = new PokerTest();
				}
				else if( testName.equalsIgnoreCase("runs") ){
					test = new RunsTest();
				}
				test.setSample( sample );
				test.run();
				// test result appending
				testModel.appendTestResult( "\n" + test.getResult() );
				if( !test.isTestPassed() )
					failed = true;
			}
		}
		// Final statement about tests
		testModel.appendTestResult( 
				"\n----------------\n" + 
				( (failed) ? "FAILED" : "SUCCEED" ) + 
				"\n----------------\n" 
				);
	}
}
