/*
 * Created on 1-feb-2005
 *
 */
package it.unibo.txs.prntest;

import java.util.Observable;

/**
 * @author patrik.suzzi@gmail.com
 *
 * This class represents the model for prn tests.
 */
public class TestModel extends Observable {
	
	// String[] that specify test names
	public final static String[] TEST_NAMES = {
			"longrun", "monobit", "poker", "runs"
	};
	
	// boolean that specify what test(s) to do.
	protected boolean [] testSelected;
	
	// specify test result.
	protected String testResult;
	
	
	/**
	 * Creates new instance of TestModel.
	 *
	 */
	public TestModel(){
		this.testSelected = new boolean[]{true, true, true, true};
		this.testResult = "no test done";
	}
	
	/**
	 * Says what to do with a test , specified by it's name, given 
	 * a boolean that specify if it's to do or not to do.
	 * @param testName String representation of the test.
	 * @param todo true if want to do test.
	 */
	public void setTest( String testName, boolean todo ){
		// declares the change to the model
		this.setChanged();
		// begins the model change
		for( int i=0; i<testSelected.length; i++) 
			// select test based on it's name
			if( TEST_NAMES[i].equalsIgnoreCase(testName) )
				// made the change in the model
				this.testSelected[i] = todo;		
		// notify the change to observer(s)
		this.notifyObservers();
	}
	
	
	/**
	 * @return Returns the testResults.
	 */
	public String getTestResult() {
		return testResult;
	}
	
	/**
	 * appends test result to existing result
	 * @param s
	 */
	public void appendTestResult( String intermediateResult ){
		// model has changed
		this.setChanged();
		// update test(s) result
		this.testResult += intermediateResult;
		// notify all the observers
		this.notifyObservers();
	}
	
	/**
	 * @param testResults The testResults to set.
	 */
	public void setTestResults(String testResults) {
		// model has changed
		this.setChanged();
		// update test(s) result
		this.testResult = testResults;
		// notify all the observers
		this.notifyObservers();
	}
	
	/**
	 * Says if the specified test is selected
	 * @param testName the name that idintify test
	 * @return true if it's selected
	 */
	public boolean isTestSelected( String testName ){
		// begins the model change
		for( int i=0; i<testSelected.length; i++) 
			// select test based on it's name
			if( TEST_NAMES[i].equalsIgnoreCase(testName) )
				return this.testSelected[i];
		// if testName passed not match return false.
		return false;
	}
	
	/**
	 * @return the number of tests selected
	 */
	public int getNTestSelected(){
		int nTest = 0;
		for( int i=0; i<this.testSelected.length; i++ ){
			if( testSelected[i] )
				nTest++;
		}
		return nTest;
	}

}
