/*
 * PokerTest.java
 *
 * Created on 24 febbraio 2004, 13.07
 */

package it.unibo.txs.prntest.tests;

/**
 *
 * @author  x9000340
 */
public class PokerTest extends PrngTest {
	
	private byte[] segments;
	
	private int[] segmValues;
	
	private double X;
	
	/** Creates a new instance of PokerTest */
	public PokerTest() {
		result = "Poker Test: ";
		segments = new byte[5000];
		segmValues = new int[16];
		X = 0;
	}
	
	/** Set contiguous segment values according to this test specifications, 
	 * @param the byte[] that contain sample random number generated
	 */
	private void setSegments(byte[] sample) {
		for (int i = 0; i < 2500; i++) {
			segments[(2 * i)] = (byte) (sample[i] & 0xF);
			segments[(2 * i) + 1] = (byte) ((sample[i] >> 4) & 0xF);
		}
	}
	
	/** set to 0 all the values of the segment occurencies */
	private void resetSegmValues() {
		for (int i = 0; i < 15; i++) {
			segmValues[i] = 0;
		}
	}
	
	/** Divide the 20,000 bit stream into 5,000 contiguous 4-bit segments. 
	 * Count and store the number of occurrences of each of the 16 possible 4-bit 
	 * values. Denote f(i) as the number of each 4-bit value i where 0 <= i <= 15.
	 * and evaluate the specified formula:
	 * X = (16/5000)*(sum(from i=0 to 15) (f(i)^2) ) - 5000
	 * the test is passed if 1.03 < X < 57.4
	 *
	 */
	public void run() {
		if (!sampleSetOk) //sample bytes not set
			return;
		setSegments(sample);
		resetSegmValues();
		
		for (int i = 0; i < 5000; i++) {
			int j = segments[i];
			if (0 > j || 15 < j)
				System.out.print("x");
			segmValues[j] += 1;
		}
		
		int sum = 0;
		int fipow2 = 0;
		for (int i = 0; i < 16; i++) {
			
			fipow2 = segmValues[i] * segmValues[i];
			sum += (fipow2);
		}
		this.X = ((16F / 5000F) * sum - 5000);
		if (1.03F < X && X < 57.4F)
			testPassed = true;
	}
	
	/** Report evolution of the operation with the monobit test.
	 * @return string representation for the actual Poker Test status.
	 *
	 */
	public String getResult() {
		String s = "\t";

		String postFix = "";

		if (sampleSetOk) {
			
			int segms = 0;
			for (int i = 0; i < 16; i++) {
				segms += segmValues[i];
			}
			
			s += " test " + ((testPassed) ? " Passed. " : " NOT Passed. ");
			s += "\t";
			postFix = "( segments: " + segms + ", X:= " + X
			+ " )";
		} else
			postFix = " sample NOT set. ";

		return result + s + postFix;
	}
	
	/** Returns the array that represent <em>f(i)</em>: the number of occurrency 
	 * for each value (0-15) that a segment can assume.
	 * @return an int[16] array epresenting <em>f(i)</em>.
	 */
	public int[] getSegmValues() {
		return this.segmValues;
	}
	
	/** Return the value of the X, such as reported in the nist Specifications
	 * @return Value of X.
	 */
	public double getX() {
		return X;
	}
	
}