/*
 * RunsTest.java
 *
 * Created on 24 febbraio 2004, 18.47
 */

package it.unibo.txs.prntest.tests;


/**
 *
 * @author  x9000340
 */
public class RunsTest extends PrngTest {
  
  private static int [] minOccurr = { 2267, 1079, 502, 223, 90 , 90  };
  private static int [] maxOccurr = { 2733, 1421, 748, 402, 223, 223 };
  
  private int sumOccurr;
  private int [] oneRunsOccurr;
  private int [] zeroRunsOccurr;
  
  /** Creates a new instance of MonobitTest */
  public RunsTest() {
    result = "Runs Test: ";
    sumOccurr = 0;
    oneRunsOccurr = new int[6];
    zeroRunsOccurr = new int[6];
  }
  
  /** reset all values of the runsOccurrencies arrays (both 0 either 1) */
  private void resetRunsOccurr(){
    sumOccurr = 0;
    for (int i=0; i<6; i++){
      oneRunsOccurr[i] = 0;
      zeroRunsOccurr[i] = 0;
    }
  }
  
  /** Gets a bit from an array of bytes, the bit position is specified with
   * the integer i.
   * @param the integer that specify the desired bit position
   */
  private boolean getBit(int i){
    // j=i/8 (integer division) et k=i%8 (module)
    // => i = (j*8)+k
    return ( ( (byte)1 )==( ( this.sample[(i/8)] >> (i%8) ) & (byte)1 ) );
  }
  
  /** save the number of occurrencies of a runs according to type of runs.
   * for example save(true, 3) adds one to the value of oneRunsOccurr[2]
   * @param logical type of the runs
   * @param number of occurrencies of the runs
   */
  private void saveRunsOccurr(boolean type, int nOccurr){
    //index of runsOccurr array = nOccurr-1 
    nOccurr = (nOccurr>6) ? 5 : (nOccurr-1);
    if (type)
      oneRunsOccurr[nOccurr]++;
    else
      zeroRunsOccurr[nOccurr]++;
  }
  
  /** test runs occurrencies according to NIST specifications 
   */
  private boolean testRunsOccurr(){
    for (int i=0; i<6; i++){
      if ( ( oneRunsOccurr[i]  < minOccurr[i] ) || 
           ( oneRunsOccurr[i]  > maxOccurr[i] ) ||
           ( zeroRunsOccurr[i] < minOccurr[i] ) || 
           ( zeroRunsOccurr[i] > maxOccurr[i] )
         )
        return false;
    }
    return true;
  }

  /** Executes the test and store the results internal at the classes.
   * after this test it's possible to observe the values of zero or one Runs
   * arraya, that misure occurrencies of runs, and it's possible to obtain a 
   * string representation for the actual status of the test
   */
  public void run() {
    resetRunsOccurr();
    
    if ( ! sampleSetOk )  //sample bytes not set
      return;
    
    boolean cur, last;
    int nOccurr = 1;
    cur = last = getBit(0);
    
    for (int i=1; i<20000; i++){
      cur = getBit(i);
      if(cur==last)
        nOccurr++;
      else{
        saveRunsOccurr(cur, nOccurr);
        sumOccurr += nOccurr; //DEBUG
        nOccurr=1;
      }
      last=cur;
    }
    //last iteration, if 
    saveRunsOccurr(cur, nOccurr);
    sumOccurr += nOccurr; //DEBUG

    testPassed = testRunsOccurr();
  }
  
  /** Report evolution of the operation with the monobit test.
   * @return string representation for the actual Monobit Test status.
   *
   */
  public String getResult() {
  	String s = "\t";

	String postFix = "";

	if (sampleSetOk) {
		
		s += " test " + ((testPassed) ? " Passed. " : " NOT Passed. ");
		s += "\t";
		postFix = "( sumOccurr. : " + sumOccurr + " )";
	} else
		postFix = " sample NOT set. ";

	return result + s + postFix;
  }  
  
}
